import struct
import re

# soul link uses X
mathops = ['i','B','X']

# these commands need repatching
function_calls = ['\x00\x01\xff\xff',
                  '\x01\x01\xff\xff',
                  '\x02\x01\xff\xff',
                  ]

coordmatch = re.compile(r"^\(\s*([0-9][0-9]*)\s*,\s*([0-9][0-9]*)\s*\)(.*)")

class ShuffleSCR:
    def __init__(self, s):
        self.tokens=[]
        ptr = 0
        while ptr < len(s):
            opcode, = s[ptr]
            if opcode == 'F':
                # function call
                length, = struct.unpack('<H',s[ptr+2:ptr+4])
            elif opcode in mathops:
                length = s.find(';',ptr+1)+1-ptr
            else:
                print("Unknown opcode '"+opcode+"' at location "+hex(ptr))
                length = s.find(';',ptr+1)+1-ptr
                lasttoken = self.tokens[-1]
                self.tokens=self.tokens[0:-1]
                length = length+len(lasttoken)
                ptr = ptr - len(lasttoken)
            token = s[ptr:ptr+length]
            self.tokens.append(token)
            ptr=ptr+length
    def dump(self):
        output = ''
        for token in self.tokens:
            if token[0xc:0x10] == '\x00\x02\xff\xff':
                # text
                output+=('text|'+token[0x28:-1]+'\r\n')
            elif token[0x0c:0x10] == '\x04\x02\xff\xff':
                # title
                output+=('title|'+token[0x29:-1]+'\r\n')
            elif token[0x0c:0x10] == '\x01\x01\x0b\x00':
                # option
                output+=('option|'+token[0x2e:-1]+'\r\n')
        return output
    def insert(self, lines, format=55):
        lineno = 0
        newtokens = []
        curoffsetold = 0
        curoffsetnew = 0
        offsetmap = {}
        for token in self.tokens:
            offsetmap[curoffsetold] = curoffsetnew
            # print lineno
            if token[0xc:0x10] == '\x00\x02\xff\xff':
                # text
                #print 'text',lines[lineno][0:5]
                assert(lines[lineno].startswith('text|'))
                #print 'text2'
                actualline = lines[lineno][5:]
                #print '---'
                #print repr(actualline)
                if format > 0:
                    remain = actualline
                    actualline = ''
                    while remain:
                        currentline = remain[0:format]
                        remain = remain[len(currentline):]
                        if not remain:
                            # end of thing
                            actualline+=currentline
                        else:
                            firsthalf = currentline[0:currentline.rfind(' ')]
                            secondhalf = currentline[len(firsthalf):]
                            secondhalf = secondhalf.lstrip(' ')
                            firsthalf = firsthalf + (format-len(firsthalf))*' '
                            actualline+=firsthalf
                            remain = secondhalf+remain
                            #print repr(firsthalf)
                            #print repr(secondhalf)
                    
                #print repr(actualline)
                newlength = len(actualline) + 0x29
                newtoken = 'F\x00'+struct.pack('<H',newlength)+token[0x04:0x28]+actualline+'\x00'
                lineno+=1
            elif token[0x0c:0x10] == '\x04\x02\xff\xff':
                # title
                assert(lines[lineno].startswith('title|'))
                actualline = lines[lineno][6:]
                newlength = len(actualline) + 0x2a
                newtoken = 'F\x00'+struct.pack('<H',newlength)+token[0x04:0x29]+actualline+'\x00'
                lineno+=1
            elif token[0x0c:0x10] == '\x01\x01\x0b\x00':
                # option
                assert(lines[lineno].startswith('option|'))
                actualline = lines[lineno][7:]
                newlength = len(actualline) + 0x2f
                newtoken = 'F\x00'+struct.pack('<H',newlength)+token[0x04:0x2e]+actualline+'\x00'
                lineno+=1
            elif token[0x0c:0x10] == '\x30\x09\x08\x00' and token[0x37] == 'T':
                # soul link onscreen
                assert(lines[lineno].startswith('soullink|'))
                actualline = lines[lineno][9:]
                xpos, ypos = struct.unpack('<LxL',token[0x2e:0x37])
                result = coordmatch.search(actualline)
                if result:
                    results = result.groups()
                    xpos = int(results[0])
                    ypos = int(results[1])
                    actualline = results[2]
                if len(actualline) > 31:
                    # truncate if too long
                    actualline = actualline[0:31]
                newlength = len(actualline) + 0x52
                oldnull = token.index('\x00',0x38)
                coords = struct.pack('<cLcLc','V',xpos,'V',ypos,'T')
                newtoken = 'F\x00'+struct.pack('<H',newlength)+token[0x04:0x2d]+coords+actualline+token[oldnull:]
                assert(len(newtoken) == newlength)
                lineno+=1
                print token[0x38:oldnull]
                print actualline
            else:
                newtoken = token
            newtokens.append(newtoken)
            curoffsetold += len(token)
            curoffsetnew += len(newtoken)
        # fix up absolute addresses
        for i in range(len(newtokens)):
            newtoken = newtokens[i]
            if newtoken[0xc:0x10] in function_calls:
                # function call
                oldoffset, = struct.unpack('<L',newtoken[0x10:0x14])
                newoffset = offsetmap[oldoffset]
                newoffsetstr = struct.pack('<L',newoffset)
                newtoken2 = newtoken[0:0x10] + newoffsetstr + newtoken[0x14:]
                newtokens[i] = newtoken2
                print 'Patching '+hex(oldoffset)+' to '+hex(newoffset)
        # copy the new tokens over
        self.tokens = newtokens
    def output(self):
        return ''.join(self.tokens)
