LEFTBRACKET = u'\u3010'
RIGHTBRACKET = u'\u3011'

def parseline(line):
    parsedline = ''
    comments = []
    leftindex = line.find('{')
    while leftindex >= 0:
        parsedline += line[0:leftindex]
        rightindex = line.index('}')
        comments.append(line[leftindex+1:rightindex])
        line = line[rightindex+1:]
        leftindex = line.find('{')
    parsedline += line
    return parsedline, comments

def import_from_sjs_file(scriptblock, filepath, options):
    fp = open(filepath,'rb')
    lines = fp.readlines()
    fp.close()
    scriptlines = []
    speakers = {}
    for line in lines:
        unicodeline = line.rstrip('\r\n').decode('shift-jisx0213')
        parsedline,comments = parseline(unicodeline)
        parsedline = parsedline.strip(' ')
        # now we have a line with comments removed
        linenum = '-empty-'
        speaker = ''
        # check for line number
        if comments:
            if comments[0][0] == '-':
                rightindex = comments[0].find('-',1)
                if rightindex >= 0:
                    numstr = comments[0][1:rightindex]
                    try:
                        linenum = str(int(numstr,16))
                    except Exception:
                        pass
        # parse speaker
        if parsedline:
            if parsedline[0] == LEFTBRACKET:
                rightindex = parsedline.find(RIGHTBRACKET)
                if rightindex > 0:
                    speaker = parsedline[1:rightindex]
                    parsedline = parsedline[rightindex+1:]
                    speakers[speaker] = True
            elif parsedline.find(': "') >= 0:
                rightindex = parsedline.find(': "')
                speaker = parsedline[0:rightindex]
                parsedline = parsedline[rightindex+2:]
                speakers[speaker] = True
            elif parsedline.find(': (') >= 0:
                rightindex = parsedline.find(': (')
                speaker = parsedline[0:rightindex]
                parsedline = parsedline[rightindex+2:]
                speakers[speaker] = True
        # add to scriptlines
        scriptlines.append((linenum,speaker,parsedline))

    # now add to our scriptblock
    choice = options['single_choice']
    if choice == 'Original Text (clear translated)':
        scriptblock.clear()
        scriptblock.type = 'ate'
        for linenum,speaker,line in scriptlines:
            sbline = scriptblock.newLine()
            sbline.otext = line
            sbline.lineNum = linenum
            sbline.speaker = speaker
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[linenum] = sbline
        for speaker in speakers:
            sbname = scriptblock.newName()
            sbname.oname = speaker
            sbname.tname = speaker
            scriptblock.namesTable[speaker] = sbname
            scriptblock.names.append(speaker)
        scriptblock.dirty = True
    elif choice == 'Original Text':
        # this is for fixing bugs in the original text
        for linenum,speaker,line in scriptlines:
            if linenum in scriptblock.linesTable:
                sbline = scriptblock.linesTable[linenum]
                sbline.otext = line
                sbline.speaker = speaker
        scriptblock.dirty = True
    elif choice == 'Translated Text':
        # this is for checking other, already-translated scripts
        for linenum,speaker,line in scriptlines:
            if linenum in scriptblock.linesTable:
                sbline = scriptblock.linesTable[linenum]
                sbline.ttext = line
                if sbline.speaker and (sbline.speaker not in scriptblock.namesTable):
                    scriptblock.names.append(sbline.speaker)
                    scriptblock.namesTable[sbline.speaker] = scriptblock.newName()
                if sbline.speaker:
                    scriptblock.namesTable[sbline.speaker].tname = speaker
        scriptblock.dirty = True

def export_to_sjs_file(scriptblock, filepath, options):
    output = ''
    choice = options['single_choice']
    if choice == 'Original Text':
        for scriptline in scriptblock.lines:
            if scriptline.lineNum != '-empty-':
                linenumstr = '{-%02x-} ' % int(scriptline.lineNum)
                speakerstr = ''
                if scriptline.speaker:
                    speakerstr = LEFTBRACKET+scriptline.speaker+RIGHTBRACKET
                linestr = linenumstr + speakerstr + scriptline.otext
                print repr(linestr)
                output+=linestr.encode('shift-jisx0213')
            output+='\r\n'
    elif choice == 'Translated Text':
        for scriptline in scriptblock.lines:
            if scriptline.lineNum != '-empty-':
                linenumstr = '{-%02x-} ' % int(scriptline.lineNum)
                speakerstr = ''
                if scriptline.speaker:
                    speaker = scriptline.speaker
                    if speaker in scriptblock.namesTable:
                        speaker = scriptblock.namesTable[speaker].tname
                    speakerstr = speaker+': '
                linestr = linenumstr + speakerstr + scriptline.ttext
                output+=linestr.encode('shift-jisx0213')
            output+='\r\n'
    else:
        raise Exception('Invalid choice: '+choice)
    f=open(filepath,'wb')
    f.write(output)
    f.close()

exports = [
    ('Shift-Jis file','.sjs',export_to_sjs_file,
     {'single_choice':
      ['Please choose which column you would like to export to a Shift-Jis file:',
       'Original Text',
       'Translated Text',
       ]
      }
     )
    ]
imports = [
    ('Shift-Jisx0213 file','.sjs',import_from_sjs_file,
     {'single_choice':
      ['Please choose which column you would like to overwrite and import the Shift-Jis file into:',
       'Original Text (clear translated)',
       'Original Text',
       'Translated Text',
       ]
      }
     )
    ]
desc = ('AIR XML script file','.xml')
