'''
Script plugins for Berkana''s Last Song game
'''

import struct
from LastSong import *

def import_from_ls_sce_dat(scriptblock, filepath, options):
    '''
    Import the script from ls_sce.dat
    '''
    # read the file
    f=open(filepath,'rb')
    encoded = f.read()
    f.close()
    # decode the file
    data = decode(encoded)
    # find where the text starts
    textoffset, = struct.unpack('<L',data[4:8])
    # get the strings
    strings = data[textoffset:].split('\x00')
    # update the script block
    scriptblock.clear()
    scriptblock.type = 'berkana'
    i=1
    for s in strings:
        sbline = scriptblock.newLine()
        sbline.otext = s.decode('shift_jis')
        sbline.ttext = s.decode('shift_jis')
        sbline.lineNum = str(i)
        sbline.speaker = ''
        scriptblock.lines.append(sbline)
        scriptblock.linesTable[str(i)]=sbline
        i+=1
    # put in the meta
    headerstr = data[0:textoffset].encode('hex')
    doc,scriptblock.meta = scriptblock.newMeta()
    scriptblock.meta.setAttribute('version','1.0')
    scriptblock.meta.setAttribute('data',headerstr)
    scriptblock.dirty=True

def export_to_ls_sce_dat(scriptblock, filepath, options):
    '''
    Export the script as a ls_sce.dat file
    '''
    # assert correctness of metadata and process it
    if scriptblock.meta.getAttribute('version') == '1.0':
        headerstr = scriptblock.meta.getAttribute('data').decode('hex')
    else:
        raise Exception('Wrong version for meta')
    strings = [x.ttext.encode('shift_jis') for x in scriptblock.lines]
    textstr = '\x00'.join(strings)
    data = headerstr+textstr
    encoded = encode(data)
    f=open(filepath,'wb')
    f.write(encoded)
    f.close()

exports=[
    ('ls_sce.dat file','.dat',export_to_ls_sce_dat,{}),
    ]

imports=[
    ('ls_sce.dat file','.dat',import_from_ls_sce_dat,{}),
    ]

desc=('Last Song Script File','.lss')
