'''
Baldr Force script format plugins
'''

import struct

nameTable = [
    'Toru',
    'Minori',
    'Ayane',
    'Tsukina',
    'Ryang',
    'Bachelor',
    'Renn',
    'Reika',
    'Kuwong',
    'Genha',
    'Yagisawa',
    'Kayla',
    'Gondou',
    'Yousuke',
    'Akira',
    'Yuuya',
    '???1',
    '???2',
    '???3',
    '']

numNames = len(nameTable)

def import_from_bin_file(scriptblock, filepath, options):
    '''
    Import the script from a .BIN file
    '''
    f=open(filepath,'rb')
    hdr = f.read(16)
    z1,z2,tablesize,scriptsize = struct.unpack('<llLL',hdr)
    if z1!=0:
        raise Exception('Not a valid .bin file')
    # blit in the table
    table = f.read(4*tablesize)
    tableHex = table.encode('hex')
    # read in the script
    script = f.read(scriptsize)
    scriptHex = script.encode('hex')
    f.close()
    # now parse the script
    pos = 0
    numstr = 0
    # each line is format (offset_in_file,size,id,spkr,string)
    lines = []
    cmd = 0
    arg = 0
    while pos<len(script):
        opcode = script[pos:pos+6]
        lastcmd = cmd
        lastarg = arg
        cmd,arg = struct.unpack('<Hl',opcode)
        if cmd == 0xb:
            # a push string command
            assert(arg>0)
            string = script[pos+6:pos+6+arg]
            assert(string[-1]=='\x00')
            string = string[0:-1]
            if lastcmd == 0xc and lastarg < numNames:
                spkr = nameTable[lastarg]
            else:
                spkr = '-'
            lines.append((pos+tablesize*4+16+2,4+arg,numstr,spkr,string))
            pos+=6+arg
            numstr+=1
        else:
            pos+=6
    # now that we have all the lines, add them to the scriptblock
    scriptblock.clear()
    scriptblock.type = 'bf'
    for offset,size,numstr,spkr,string in lines:
        sbline = scriptblock.newLine()
        sbline.otext = string.decode('shift_jis')
        if spkr == '-':
            sbline.ttext = sbline.otext
        sbline.lineNum = str(numstr)
        sbline.speaker = spkr
        scriptblock.lines.append(sbline)
        scriptblock.linesTable[numstr] = sbline
    # TODO: store the meta stuff
    scriptblock.dirty = True

def export_to_binary_patch(scriptblock, filepath, options):
    '''
    Export the script to a binary patch file
    '''
    pass

def export_to_text_file(scriptblock, filepath, options):
    '''
    Export the script to a text file
    '''
    # open the file
    f = open(filepath,'wb')
    for line in scriptblock.lines:
        if line.lineNum >= 0:
            textline = ''
            speaker = line.speaker
            if speaker in scriptblock.namesTable:
                speaker = scriptblock.namesTable[speaker].tname
            if speaker:
                textline+=(speaker+': ')
            textline+=(line.ttext)
            if line.comment.strip():
                textline+=(' //'+line.comment.strip())
            f.write(textline+'\n')
        else:
            f.write('\n')
    f.close()    

# the following must be present in each plugin file

exports=[
    ('Binary patch','.bpf',export_to_binary_patch,{}),
    ('Text file','.txt',export_to_text_file,{}),
    ]

imports=[
    ('BIN file','.bin',import_from_bin_file,{}),
    ]

desc=('Baldr Force script file','.xml')
