# Plugin module for manipulating fixed length strings in files

import imp
import fixed
import sys
import struct

class Error(Exception):
    pass

_bin_import_options = {
    'select_files':[
    ('Please choose the configuration file',
     '','',
     'Configuration file (*.cfg)|*.cfg',False)
    ]
    }

def import_from_bin_file(scriptblock, filepath, options):
    '''
    Import the script from a binary file
    '''
    config_path = options['select_files'][0][0]
    # load the config
    config = imp.load_source('config', config_path)
    # load the binary file
    fr = fixed.FixedReplacer(config)
    fr.load(open(filepath,'rb'))
    # reset script block
    scriptblock.clear()
    scriptblock.type = 'fls'
    # load the strings
    for s,offset,length,i,cur_id,encoding in fr.strings:
        sbline = scriptblock.newLine()
        sbline.otext = s
        sbline.ttext = s
        sbline.lineNum = '('+hex(offset)+','+hex(length)+','+repr(encoding)+')'
        sbline.speaker = str(cur_id)
        scriptblock.lines.append(sbline)
        scriptblock.linesTable[sbline.lineNum] = sbline
    scriptblock.dirty = True    

_text_export_options = {
    'single_choice':
    ['Please choose which column you would like to export to a text file:',
     'Original Text',
     'Translated Text',
     ]
    }

def export_to_text_file(scriptblock, filepath, options):
    '''
    Export the script to a text file
    '''
    # open the file
    if options['single_choice'] == 'Original Text':
        f = open(filepath,'wb')
        for line in scriptblock.lines:
            address_string = line.lineNum
            actual_text = line.otext
            print >> f, address_string + ',' + actual_text.encode('utf-8')
        f.close()
    elif options['single_choice'] == 'Translated Text':
        f = open(filepath,'wb')
        for line in scriptblock.lines:
            address_string = line.lineNum
            actual_text = line.ttext
            print >> f, address_string + ',' + actual_text.encode('utf-8')
        f.close()

_bin_export_options = {
    'select_files':[
    ('Please choose the original binary file',
     '','',
     'Binary file (*.*)|*.*',False)
    ]
    }

def export_to_bin_file(scriptblock, filepath, options):
    '''
    Write a new binary file with the new strings
    '''
    original_bin_path = options['select_files'][0][0]
    # parse the patch
    parsed_lines = []
    error_lines = []
    for line in scriptblock.lines:
        offset, length, encoding = eval(line.lineNum)
        actual_text = line.ttext
        replacement = actual_text.encode(encoding)
        if len(replacement) > length:
            error_lines.append(line.lineNum+','+actual_text)
            print >> sys.stderr, line.lineNum+','+actual_text
            continue
        print offset, length, replacement
        replacement = replacement + ('\x00'*(length-len(replacement)))
        parsed_lines.append((offset, replacement))
    if error_lines:
        print >> sys.stderr, error_lines
        raise Error('Text too long on the following lines:\n\t'+'\n\t'.join(error_lines))
    # sort the lines by increasing offset
    parsed_lines.sort(lambda x,y:cmp(x[0],y[0]))
    # open the original file
    fin = open(original_bin_path,'rb')
    # open the output file
    fout = open(filepath,'wb')
    for offset, replacement in parsed_lines:
        if offset > fout.tell():
            # fill in missing data
            amount = offset-fout.tell()
            fin.seek(fout.tell())
            block = fin.read(amount)
            if len(block) != amount:
                raise Error('EOF encountered in input file prematurely')
            fout.write(block)
        elif offset < fout.tell():
            # went too far
            fout.seek(offset)
        fout.write(replacement)
    # write the rest of the file
    fin.seek(fout.tell())
    fout.write(fin.read())
    # done
    fin.close()
    fout.close()

# the following must be present in each plugin file
exports=[
    ('Binary file','.*',export_to_bin_file,_bin_export_options),
    ('Text file','.txt',export_to_text_file,_text_export_options),
    ]

imports=[
    ('Binary file','.*',import_from_bin_file,_bin_import_options),
    ]

desc=('Fixed length strings','.fls')
