# Plugin module for manipulating location aware strings in files

import imp
import pe
import sys
import struct

def _read_string(data, offset, encoding):
    endoffset = data.find('\x00',offset)
    s=data[offset:endoffset]
    # decode the shift jis
    s=s.decode(encoding)
    # replace tab with \t
    s=s.replace('\x09','\\t')
    s=s.replace('\x0a','\\n')
    s=s.replace('\x0d','\\r')
    return s

def _reparse_string(s, encoding):
    s=s.replace('\\t','\x09')
    s=s.replace('\\n','\x0a')
    s=s.replace('\\r','\x0d')
    s=s.encode(encoding)+'\x00'
    return s

_exe_import_options = {
    'select_files':[
    ('Please choose the configuration file',
     '','',
     'Configuration file (*.py)|*.py',False)
    ]
    }

def import_from_exe_file(scriptblock, filepath, options):
    '''
    Import the script from a .exe file
    '''
    config_path = options['select_files'][0][0]
    # load the config
    config = imp.load_source('config', config_path)
    # load the exe
    exe_file = pe.PESections(filepath)
    # get memory view
    memory_view = exe_file.get_view()
    # reset script block
    scriptblock.clear()
    scriptblock.type = 'las'
    for pointer_location in config.string_pointers:
        try:
            # check to see if it's a multi-reference
            at_locations = [struct.unpack('<L',memory_view[x:x+4])[0] for x in pointer_location]
            if 'pedantic' in config.__dict__ and config.pedantic:
                for at_location, p_location in zip(at_locations,pointer_location):
                    if at_location != at_locations[0]:
                        raise Error('Inconsistent multi-reference: mem[%X]=%X, mem[%X]=%X' % (p_location, at_location, pointer_location[0], at_loactions[0]))
            p_locations = pointer_location
        except TypeError:
            #print >> sys.stderr, hex(pointer_location)
            # find where this pointer points to
            at_locations = struct.unpack('<L',memory_view[pointer_location:pointer_location+4])
            p_locations = [pointer_location]
        #print >> sys.stderr, '@'+hex(at_location)
        # find the section it points to
        print at_locations
        section_name, section_offset = exe_file.address_table.lookup(at_locations[0])
        # dump the string
        exe_string = _read_string(exe_file.sections[exe_file.section_lookup[section_name]]['Data'],section_offset,config.encoding)
        # add the line
        sbline = scriptblock.newLine()
        sbline.otext = exe_string
        sbline.ttext = exe_string
        sbline.lineNum = ','
        sbline.lineNum = ','.join([hex(x) for x in p_locations]) + ',' + hex(at_locations[0])
        sbline.speaker = ''
        scriptblock.lines.append(sbline)
        scriptblock.linesTable[sbline.lineNum] = sbline
    # save the configuration data
    doc,scriptblock.meta = scriptblock.newMeta()
    scriptblock.meta.setAttribute('encoding',config.encoding)
    scriptblock.dirty = True

_text_export_options = {
    'single_choice':
    ['Please choose which column you would like to export to a text file:',
     'Original Text',
     'Translated Text',
     ]
    }

def export_to_text_file(scriptblock, filepath, options):
    '''
    Export the script to a text file
    '''
    # open the file
    if options['single_choice'] == 'Original Text':
        f = open(filepath,'wb')
        for line in scriptblock.lines:
            address_string = line.lineNum
            actual_text = line.otext
            print >> f, address_string + ',' + actual_text.encode('utf-8')
        f.close()
    elif options['single_choice'] == 'Translated Text':
        f = open(filepath,'wb')
        for line in scriptblock.lines:
            address_string = line.lineNum
            actual_text = line.ttext
            print >> f, address_string + ',' + actual_text.encode('utf-8')
        f.close()

def _exe_string_patch(exefile, parsed_lines):
    new_sections = {}
    new_lines = []
    for parsed_line in parsed_lines:
        pointer_locations = parsed_line[0:-2]
        data_location = parsed_line[-2]
        new_string = parsed_line[-1]
        # first, find out what section the data lies in
        section_name, section_offset = exefile.address_table.lookup(data_location)
        section = exefile.sections[exefile.section_lookup[section_name]]
        # find the characteristics and merge into new section
        characteristics = section['Characteristics']
        if characteristics not in new_sections:
            new_sections[characteristics] = ''
        # set the new data
        new_line = pointer_locations, len(new_sections[characteristics]), characteristics
        new_sections[characteristics] += new_string
        new_lines.append(new_line)
    # add the new sections
    i = 0
    new_offsets = {}
    for characteristics in new_sections:
        data = new_sections[characteristics]
        # round up to next alignment
        data += '\x00' * ((-len(data)) % exefile.opt_header['FileAlignment'])
        name = '.new%02d' % i
        virtual_address = exefile.address_table.len() - exefile.opt_header['ImageBase']
        virtual_address += (-virtual_address)%(exefile.opt_header['SectionAlignment'])
        # round up to the next alignment
        virtual_size = len(data) + (-len(data))%(exefile.opt_header['SectionAlignment'])
        i += 1
        # add the section
        # print hex(virtual_address),hex(virtual_size)
        exefile.add_section(name,virtual_address,virtual_size,characteristics,data)
        # remember the offset
        new_offsets[characteristics] = virtual_address + exefile.opt_header['ImageBase']
    # repatch the code
    memory_view = exefile.get_view()
    for pointer_locations, offset, characteristics in new_lines:
        for pointer_location in pointer_locations:
            memory_view[pointer_location:pointer_location+4] = struct.pack('<L',offset+new_offsets[characteristics])

_exe_export_options = {
    'select_files':[
    ('Please choose the original exe file',
     '','',
     'Windows 32-bit executable (*.exe,*.dll)|*.exe;*.dll',False)
    ]
    }

def export_to_exe_file(scriptblock, filepath, options):
    '''
    Write a new exe file with the new strings
    '''
    encoding = scriptblock.meta.getAttribute('encoding')
    original_exe_path = options['select_files'][0][0]
    # parse the patch
    parsed_lines = []
    for line in scriptblock.lines:
        address_string = line.lineNum
        actual_text = line.ttext
        parsed_lines.append(eval(address_string) + (_reparse_string(actual_text, encoding),))
    # first, load the exe
    exefile = pe.PESections(original_exe_path)
    # save as the new exe
    exefile.save(filepath)
    # reload the new exe
    exefile = pe.PESections(filepath)
    # then, patch the strings
    _exe_string_patch(exefile, parsed_lines)
    # write the exe
    exefile.save(filepath)

# the following must be present in each plugin file
exports=[
    ('Windows 32-bit executable','.exe',export_to_exe_file,_exe_export_options),
    ('Windows 32-bit executable library','.dll',export_to_exe_file,_exe_export_options),
    ('Text file','.txt',export_to_text_file,_text_export_options),
    ]

imports=[
    ('Windows 32-bit executable','.exe',import_from_exe_file,_exe_import_options),
    ('Windows 32-bit executable library','.dll',import_from_exe_file,_exe_import_options),
    ]

desc=('Location aware script','.las')
