'''
Script plugins for CIRCUS''s .mes files
'''

import struct
import mes
import textwrap

class Error(Exception):
    pass

def import_from_mes(scriptblock, filepath, options):
    '''
    Import the script from a .mes file
    '''
    # read the file
    mesfile = mes.MES()
    mesfile.load(filepath)
    # load the strings
    scriptblock.clear()
    scriptblock.type = 'mes'
    i=1
    curspeaker = ''
    for token in [x for x in mesfile.tokens if x[0] in mesfile.textopcodes]:
        decode = mesfile.textopcodes[0] != token[0]
        s = token[1:-1]
        if decode:
            s = ''.join([chr((ord(x)+0x20)&0xff) for x in s])
        try:
            s = s.decode('cp932')
        except Exception:
            print >> sys.stderr, token.encode('hex')
            print >> sys.stderr, s.encode('hex')
            sys.exit(-1)
        if token[0] == mesfile.textopcodes[0]:
            # scene descriptor
            s = s.replace('\x0a','\\n')
            sbline = scriptblock.newLine()
            sbline.otext = s
            sbline.ttext = s
            sbline.lineNum = str(i)
            sbline.speaker = '(scene)'
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)] = sbline
            i+=1
        elif token[0] == mesfile.textopcodes[1]:
            # speaker
            curspeaker = s
        elif token[0] == mesfile.textopcodes[2]:
            # line
            sbline = scriptblock.newLine()
            sbline.otext = s
            sbline.ttext = s
            sbline.lineNum = str(i)
            sbline.speaker = curspeaker
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)] = sbline
            i+=1
        elif token[0] == mesfile.textopcodes[3]:
            # choice
            sbline = scriptblock.newLine()
            sbline.otext = s
            sbline.ttext = s
            sbline.lineNum = str(i)
            sbline.speaker = '(choice)'
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)] = sbline
            i+=1
    scriptblock.dirty=True

_mes_export_options = {
    'select_files':[
    ('Please choose the original .mes file',
     '','',
     'CIRCUS .mes file (*.mes)|*.mes',False)
    ]
    }

def export_to_mes(scriptblock, filepath, options):
    '''
    Export the script to a .mes file
    '''
    # read the original file
    original_mes_path = options['select_files'][0][0]
    mesfile = mes.MES()
    mesfile.load(original_mes_path)
    # create a word wrapper
    wrapper = textwrap.TextWrapper(width=mesfile.linewidth,
                                   break_long_words=False)
    # load the strings
    cur_line_index = 0
    new_tokens = []
    for token in mesfile.tokens:
        if token[0] not in mesfile.textopcodes:
            new_tokens.append(token)
            continue
        decode = mesfile.textopcodes[0] != token[0]
        if token[0] == mesfile.textopcodes[0]:
            # scene descriptor
            if scriptblock.lines[cur_line_index].speaker != '(scene)':
                raise Error('Script file mismatch: (scene)')
            s = scriptblock.lines[cur_line_index].ttext
            s = s.encode('cp932')
            cur_line_index += 1
        elif token[0] == mesfile.textopcodes[1]:
            # speaker
            speaker = scriptblock.lines[cur_line_index].speaker
            if speaker in scriptblock.namesTable:
                speaker = scriptblock.namesTable[speaker].tname
            s = speaker.encode('cp932')
        elif token[0] == mesfile.textopcodes[2]:
            # line
            line = scriptblock.lines[cur_line_index].ttext
            line = line.encode('cp932')
            line_parts = line.split('\\n')
            new_line_parts = []
            for line_part in line_parts:
                new_line_part = wrapper.fill(line_part)
                new_line_parts.append(new_line_part)
            new_line = '\x0a'.join(new_line_parts)
            s = new_line
            cur_line_index += 1
        elif token[0] == mesfile.textopcodes[3]:
            # choice
            if scriptblock.lines[cur_line_index].speaker != '(choice)':
                raise Error('Script file mismatch: (choice)')
            s = scriptblock.lines[cur_line_index].ttext
            s = s.encode('cp932')
            cur_line_index += 1
        if decode:
            # replace spaces with other stuff
            s = s.replace('\x20','\x00')
            s = ''.join([chr((ord(x)-0x20)&0xff) for x in s])
        new_token = token[0] + s + '\x00'
        new_tokens.append(new_token)
    mesfile.tokens = new_tokens
    mesfile.save(filepath)

exports=[
    ('CIRCUS .mes file','.mes',export_to_mes,_mes_export_options),
    ]

imports=[
    ('CIRCUS .mes file','.mes',import_from_mes,{}),
    ]

desc=('XML script file for CIRCUS games','.xml')

# name override (use this if filename is different from script tag)
name = 'mes'
