'''
Script plugins for Navel''s earlier games, like Shuffle and Soul Link
'''

import struct
import ShuffleLZ
from ShuffleSCR import *

def import_from_bin_file(scriptblock, filepath, options):
    '''
    Import the contents of a .BIN script file
    '''
    fp = open(filepath,'rb')
    s = fp.read()
    fp.close()
    assert(s[0:12]==' RCS\x01\x00\x00\x00\x01\x00\x00\x00')
    decompSize, = struct.unpack('<L',s[0x10:0x14])
    assert(s[0x80:0x84] == '  ZL')
    decompSize2,inputSize = struct.unpack('<LL',s[0x84:0x8C])
    assert(decompSize == decompSize2)
    # now decompress the file
    lzblock = s[0x90:0x90+inputSize]
    assert(len(lzblock)==inputSize)
    outblock = ShuffleLZ.decompress(lzblock)
    assert(len(outblock)==decompSize)
    # load up and parse the tokens
    scr=ShuffleSCR(outblock)
    # now, let's do the lines, shall we?
    speaker=''
    scriptblock.clear()
    scriptblock.type = 'navel1'
    i=1
    for token in scr.tokens:
        if token[0xc:0x10] == '\x00\x02\xff\xff':
            # text
            s=token[0x28:-1]
            if s[0:2] == '\x81\x40':
                s='|'+s[2:]
            sbline = scriptblock.newLine()
            sbline.otext = s.decode('shift_jis')
            sbline.ttext = ''
            sbline.lineNum = str(i)
            i+=1
            sbline.speaker = speaker
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)]=sbline
            speaker=''
        elif token[0x0c:0x10] == '\x04\x02\xff\xff':
            # speaker
            speaker = token[0x29:-1].decode('shift_jis')
            if speaker not in scriptblock.namesTable:
                sbname = scriptblock.newName()
                sbname.oname = speaker
                sbname.tname = speaker
                scriptblock.names.append(speaker)
                scriptblock.namesTable[speaker]=sbname
        elif token[0x0c:0x10] == '\x01\x01\x0b\x00':
            # option
            s=token[0x2e:-1]
            #if s[0:2] == '\x81\x40':
            #    s='|'+s[2:]
            sbline = scriptblock.newLine()
            sbline.otext = s.decode('shift_jis')
            sbline.ttext = ''
            sbline.lineNum = str(i)
            i+=1
            sbline.speaker = 'option'
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)]=sbline
        elif token[0x0c:0x10] == '\x30\x09\x08\x00' and token[0x37] == 'T':
            # soul link onscreen text
            s = token[0x38:]
            null_loc = s.find('\x00')
            s = s[0:null_loc]
            if s[0:2] == '\x81\x40':
                s='|'+s[2:]
            xpos, ypos = struct.unpack('<LxL',token[0x2e:0x37])
            sbline = scriptblock.newLine()
            sbline.otext = ('(%d,%d)'%(xpos,ypos))+s.decode('shift_jis')
            sbline.ttext = ''
            sbline.lineNum = str(i)
            i+=1
            sbline.speaker = 'soullink'
            scriptblock.lines.append(sbline)
            scriptblock.linesTable[str(i)]=sbline

    # put in the meta
    fullbin = outblock.encode('hex')
    doc,scriptblock.meta = scriptblock.newMeta()
    scriptblock.meta.setAttribute('version','1.0')
    scriptblock.meta.setAttribute('data',fullbin)
    scriptblock.dirty=True

def export_to_bin_file(scriptblock, filepath, options):
    '''
    Export the script to a .BIN file
    '''
    # line_size = 71 # full soul link dialog box (width=720)
    line_size = 58 # soul link log box (width=590)
    if scriptblock.meta.getAttribute('version') != '1.0':
        raise Exception('Wrong version of meta data')
    fullbin = scriptblock.meta.getAttribute('data').decode('hex')
    scr=ShuffleSCR(fullbin)
    print 'Script loaded'
    # convert the script into a set of lines
    lines = []
    for line in scriptblock.lines:
        speaker = line.speaker
        if not line.ttext:
            text = line.otext
        else:
            text = line.ttext
        text = text.encode('shift_jis')
        #if text[0] == '|':
        #    text = '\x81\x40' + text[1:]
        if speaker == 'option':
            # add an option entry
            lines.append('option|'+text)
        elif speaker == 'soullink':
            # add a soul link entry
            lines.append('soullink|'+text)
        elif speaker == '':
            # non speaker text
            lines.append('text|'+text)
        else:
            # add a real entry
            if speaker in scriptblock.namesTable:
                speaker = scriptblock.namesTable[speaker].tname
            speaker = speaker.encode('shift_jis')
            lines.append('title|'+speaker)
            lines.append('text|'+text)
    # insert it
    for line in lines:
        print line
    print 'Script processed'
    scr.insert(lines,line_size)
    print 'Script inserted'
    output = scr.output()
    outputCompressed = ShuffleLZ.compress(output)
    print 'Script compressed'
    decompSize = len(output)
    compSize = len(outputCompressed)
    hdr = ' RCS\x01\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00'
    hdr += struct.pack('<L',decompSize) + '\x00'*12
    hdr += '\x00' * 0x60
    hdr += '  ZL'+struct.pack('<L',decompSize) + struct.pack('<L',compSize) + '\x00' * 4
    fp = open(filepath,'wb')
    fp.write(hdr+outputCompressed)
    fp.close()
    print 'Script written'

    
exports=[
    ('Navel script binary file','.BIN',export_to_bin_file,{}),
    ]

imports=[
    ('Navel script binary file','.BIN',import_from_bin_file,{}),
    ]

desc=('Navel script xml file','.ns1')
